@@ -7,7 +7,6 @@
  */
 
 package org.redmars.wadc;
-import java.util.*;
 
 class AutoRule {
   AutoRule next;
@@ -20,10 +19,10 @@ class AutoRule {
         if(w==0 || w==sw || (w<0 && sw>(-w))) {
           if(f==0 || f==sf || (f<0 && sf>(-f))) {
             return tex;
-          };
-        };
-      };
-    };
+          }
+        }
+      }
+    }
     if(next!=null) return next.retexture(t, sh, sw, sf);
     if(t.equals("C") || t.equals("F")) return "SLIME13";    // just incase
     return "METAL";
