@@ -7,11 +7,9 @@
  */
 
 package org.redmars.wadc;
-import java.util.*;
 
 class Builtin {
   static Exp n = new Int(0);
-  Vector args = null;
   int nargs;
   Exp eval() { return null; }
   Exp eval(Exp a) { return null; }
@@ -20,5 +18,6 @@ class Builtin {
   Exp eval(Exp a, Exp b, Exp c, Exp d) { return null; }
   Exp eval(Exp a, Exp b, Exp c, Exp d, Exp e) { return null; }
   Exp eval(Exp a, Exp b, Exp c, Exp d, Exp e, Exp f) { return null; }
+  Exp eval(Exp a, Exp b, Exp c, Exp d, Exp e, Exp f, Exp g, Exp h) { return null; }
 };
 
