@@ -10,7 +10,6 @@ package org.redmars.wadc;
 import java.util.*;
 
 class Choice extends Exp {
-
   static long seed;
   static Random rnd = new Random();
 
@@ -19,10 +18,17 @@ class Choice extends Exp {
       rnd.setSeed(s);
   }
 
-  Vector v = new Vector();
-  void add(Exp e) { v.addElement(e); }
+  List<Exp> v = new ArrayList<>();
+
+  void add(Exp e) {
+    v.add(e);
+  }
+
   Exp replace(Vector n, Vector r) {
-    return ((Exp)v.elementAt(Math.abs(rnd.nextInt())%v.size())).replace(n,r);
+    return (v.get(Math.abs(rnd.nextInt()) % v.size())).replace(n,r);
+  }
+
+  String show() {
+    return "[choice]";
   }
-  String show() { return "[choice]"; };
 }
